'use strict';

/* Controllers */

angular.module('tutEngine.controllers', []).
  controller('AppCtrl', ['$scope', '$http', '$location', function($scope, $http, $location) {
    // model contains variables for use in the tutorial text
    $scope.model = {
      nextButton: "<button class='page-button'>»</button>",
      prevButton: "<button class='page-button'>«</button>",

      // LMS properties
      examTitle: window.parent.title || "{examTitle}",
      timeAllowed: window.parent.totalTime || "{timeAllowed}",
      numQuestions: window.parent.questionCount || "{numQuestions}",
      programTitle: window.parent.programTitle || "{programTitle}",
      productName: window.parent.productName || "{productName}"
    };

    // get config
    $http.get(window.tutConfigPath).success(function(data, status) {
      // convert config XML to JSON and tidy up the <pages> array
      var config = $.xml2json(data);
      config.pages = config.pages.page;
      delete config.pages.page;

      // add config data to the scope
      angular.extend($scope, {
        config: config
      });

      // get strings
      $http.get($scope.config.strings).success(function(data, status) {
        // replace model variables in XML source
        for (var prop in $scope.model) {
          data = data.split("{" + prop + "}").join($scope.model[prop]);
        }

        // convert XML to JSON
        $scope.strings = $.xml2json(data);

        // set first page
        $scope.setPage(0);
      }).error(function() {
        alert("Error loading strings file: " + $scope.config.strings);
      })
    }).error(function() {
      alert("Error loading config file: " + window.configPath);
    });
      
    $scope.setPage = function(pageIndex) {
      $scope.currentPageIndex = pageIndex;
      $scope.currentPage = $scope.config.pages[pageIndex];

      // clear selected state from all page buttons
      for (var i = 0; i < $scope.config.pages.length; i++) {
        $("#pagebtn-" + i).removeClass("active");
      }

      // set the selected button to "active"
      $("#pagebtn-" + pageIndex).addClass("active");

      // if we're on the last page, display the "begin exam" button
      if ($scope.currentPageIndex == $scope.config.pages.length - 1) {
        $("#nextbtn").text($scope.strings.begin_button);
      }
      else {
        $("#nextbtn").text("»");
      }
    };

    $scope.prevPage = function() {
      if ($scope.currentPageIndex > 0)
        $scope.setPage($scope.currentPageIndex - 1);
    }

    $scope.nextPage = function() {
      if ($scope.currentPageIndex < $scope.config.pages.length - 1) {
        $scope.setPage($scope.currentPageIndex + 1);
      }
      else if ($scope.currentPageIndex == $scope.config.pages.length - 1) {
        // this function should be available in the LMS iframe's parent
        if (window.parent.launchExam)
          window.parent.launchExam();
        else
          alert("Error: Cannot find exam launcher routine.");
      }
    }
  }]);