var tutEngine = angular.module('tutEngine', []);

tutEngine.controller('AppCtrl', ['$scope', '$http', function($scope, $http) {
  // model contains variables for use in the tutorial text
  $scope.model = {
    nextButton: "<button class='page-button'>»</button>",
    prevButton: "<button class='page-button'>«</button>",

    // LMS properties
    examTitle: window.parent.title || "{examTitle}",
    timeAllowed: window.parent.totalTime || "{timeAllowed}",
    numQuestions: window.parent.questionCount || "{numQuestions}",
    programTitle: window.parent.programTitle || "{programTitle}",
    productName: window.parent.productName || "{productName}"
  };

  // preload resources (examPath will be undefined if no courseID data was found)
  if (window.examPath) {
    // start preloader spinner
    var opts = {lines:13,length:7,width:2,radius:10,corners:1,rotate:0,color:"#3D89B9",speed:1.5,trail:60,shadow:false,hwaccel:false,className:"spinner",zIndex:2e9,top:"auto",left:"auto"}
    var spinner = new Spinner(opts).spin(document.getElementById('preloader-spin'));

    var examResourceFilePath = window.examPath + "assets/resources.xml";
    //console.log("examResourceFilePath:", examResourceFilePath);
    $http.get(examResourceFilePath).success(function(data, status) {
      //console.log("data:", data);

      // convert resources XML to JSON
      var resources = $.xml2json(data).resource;
      //console.log("resources:", resources);

      var queue = new createjs.LoadQueue(true, window.examPath);
      queue.on("fileload", function(event) {
        //console.log("file preloaded");
      }, this);
      queue.on("complete", function(event) {
        spinner.stop();
//          console.log("all files preloaded");
      }, this);

      if (angular.isArray(resources)) {
        angular.forEach(resources, function(value, key) {
          //console.log("value: " + value);
          //console.log("queuing: " + value.path);
          queue.loadFile(value.path);
        });
      }
      else {
        queue.loadFile(resources.path);
      }
    }).error(function() {
      spinner.stop();
      console.log("Error loading resources file: " + examResourceFilePath);
    });
  }

  // get config
  $http.get(window.tutConfigPath).success(function(data, status) {
    // convert config XML to JSON and tidy up the <pages> array
    var config = $.xml2json(data);
    config.pages = config.pages.page;
    delete config.pages.page;

    // add config data to the scope
    angular.extend($scope, {
      config: config
    });

    // inject CSS
    if ($scope.config.css) {
      $('head').append('<link rel="stylesheet" href="' + $scope.config.css + '" type="text/css" />');
    }

    // get strings
    $http.get($scope.config.strings).success(function(data, status) {
      // replace model variables in XML source
      for (var prop in $scope.model) {
        data = data.split("{" + prop + "}").join($scope.model[prop]);
      }

      // convert XML to JSON
      $scope.strings = $.xml2json(data);

      // set first page
      $scope.setPage(0);
    }).error(function() {
      alert("Error loading strings file: " + $scope.config.strings);
    })
  }).error(function() {
    alert("Error loading config file: " + window.configPath);
  });

  $scope.setPage = function(pageIndex) {
    $scope.currentPageIndex = pageIndex;
    $scope.currentPage = $scope.config.pages[pageIndex];

    // clear selected state from all page buttons
    for (var i = 0; i < $scope.config.pages.length; i++) {
      $("#pagebtn-" + i).removeClass("active");
    }

    // set the selected button to "active"
    $("#pagebtn-" + pageIndex).addClass("active");

    // if we're on the last page, display the "begin exam" button
    if ($scope.currentPageIndex == $scope.config.pages.length - 1) {
      $("#nextbtn").text($scope.strings.begin_button);
    }
    else {
      $("#nextbtn").text("»");
    }
  };

  $scope.prevPage = function() {
    if ($scope.currentPageIndex > 0) {
      $scope.setPage($scope.currentPageIndex - 1);
    }
  };

  $scope.nextPage = function() {
    if ($scope.currentPageIndex < $scope.config.pages.length - 1) {
      $scope.setPage($scope.currentPageIndex + 1);
    }
    else if ($scope.currentPageIndex == $scope.config.pages.length - 1) {
      // this function should be available in the LMS iframe's parent
      if (window.parent.launchExam)
        window.parent.launchExam();
      else
        alert("Error: Cannot find exam launcher routine.");
    }
  }
}]);

tutEngine.directive('ngBindHtmlUnsafe', ['$sce', function($sce) {
  return {
    scope: {
      ngBindHtmlUnsafe: '='
    },
    template: "<div ng-bind-html='trustedHtml'></div>",
    link: function($scope, iElm, iAttrs, controller) {
      $scope.updateView = function() {
        $scope.trustedHtml = $sce.trustAsHtml($scope.ngBindHtmlUnsafe);
      };

      $scope.$watch('ngBindHtmlUnsafe', function(newVal, oldVal) {
        $scope.updateView(newVal);
      });
    }
  };
}]);